/*
 * ------------------------------------------------------------------------
 *
 *  Copyright (C) 2003 - 2013
 *  University of Konstanz, Germany and
 *  KNIME GmbH, Konstanz, Germany
 *  Website: http://www.knime.org; Email: contact@knime.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 3, as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses>.
 *
 *  Additional permission under GNU GPL version 3 section 7:
 *
 *  KNIME interoperates with ECLIPSE solely via ECLIPSE's plug-in APIs.
 *  Hence, KNIME and ECLIPSE are both independent programs and are not
 *  derived from each other. Should, however, the interpretation of the
 *  GNU GPL Version 3 ("License") under any applicable laws result in
 *  KNIME and ECLIPSE being a combined program, KNIME GMBH herewith grants
 *  you the additional permission to use and propagate KNIME together with
 *  ECLIPSE with only the license terms in place for ECLIPSE applying to
 *  ECLIPSE and the GNU GPL Version 3 applying for KNIME, provided the
 *  license terms of ECLIPSE themselves allow for the respective use and
 *  propagation of ECLIPSE together with KNIME.
 *
 *  Additional permission relating to nodes for KNIME that extend the Node
 *  Extension (and in particular that are based on subclasses of NodeModel,
 *  NodeDialog, and NodeView) and that only interoperate with KNIME through
 *  standard APIs ("Nodes"):
 *  Nodes are deemed to be separate and independent programs and to not be
 *  covered works.  Notwithstanding anything to the contrary in the
 *  License, the License does not apply to Nodes, you are not required to
 *  license Nodes under the License, and you are granted a license to
 *  prepare and propagate Nodes, in each case even if such Nodes are
 *  propagated with or for interoperation with KNIME.  The owner of a Node
 *  may freely choose the license terms applicable to such Node, including
 *  when such Node is propagated with or for interoperation with KNIME.
 * ---------------------------------------------------------------------
 *
 * Created on 16.08.2013 by Kilian Thiel
 */
package org.knime.example;

import java.io.File;
import java.io.IOException;

import org.knime.core.data.DataRow;
import org.knime.core.data.DataTableSpec;
import org.knime.core.data.RowIterator;
import org.knime.core.node.BufferedDataTable;
import org.knime.core.node.CanceledExecutionException;
import org.knime.core.node.ExecutionContext;
import org.knime.core.node.ExecutionMonitor;
import org.knime.core.node.InvalidSettingsException;
import org.knime.core.node.NodeModel;
import org.knime.core.node.NodeSettingsRO;
import org.knime.core.node.NodeSettingsWO;
import org.knime.core.node.defaultnodesettings.SettingsModelBoolean;
import org.knime.ext.textprocessing.data.DocumentValue;
import org.knime.ext.textprocessing.nodes.tagging.DocumentTagger;
import org.knime.ext.textprocessing.util.DataTableSpecVerifier;
import org.knime.ext.textprocessing.util.DocumentDataTableBuilder;

/**
 * This is the model implementation of SentimentTagger.
 * Example custom sentiment tag set and tagger.
 *
 * @author Kilian Thiel, KNIME.com, Zurich, Switzerland
 */
public class SentimentTaggerNodeModel extends NodeModel {
    
	private SettingsModelBoolean m_unmodifiableModel = SentimentTaggerNodeDialog.createSetUnmodifiableModel();
	
	private int m_docColIndex = -1;
	
	private DocumentDataTableBuilder m_dtBuilder;

	/**
	 * Creates new instance of {@link SentimetTaggerNodeModel} which adds
	 * sentiment tags to terms of documents.
	 */
	public SentimentTaggerNodeModel() {
		super(1, 1);
		m_dtBuilder = new DocumentDataTableBuilder();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected DataTableSpec[] configure(final DataTableSpec[] inSpecs)
			throws InvalidSettingsException {
		DataTableSpecVerifier verfier = new DataTableSpecVerifier(inSpecs[0]);
		verfier.verifyDocumentCell(true);
		m_docColIndex = verfier.getDocumentCellIndex();
		
		return new DataTableSpec[] { m_dtBuilder.createDataTableSpec() };
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected BufferedDataTable[] execute(final BufferedDataTable[] inData,
			final ExecutionContext exec) throws Exception {
		// create tagger instance with unmodifiable setting
		DocumentTagger tagger = new SentimentTagger(m_unmodifiableModel.getBooleanValue());
		
		// prepare data table builder to be able to add tagged documents
		m_dtBuilder.openDataTable(exec);
		
		// prepare row iterator and row counts
		RowIterator it = inData[0].iterator();
		int rowCount = inData[0].getRowCount();
		int currDoc = 1;
		while (it.hasNext()) {
			// check if process has been canceled
			exec.checkCanceled();
			
			// set progress
			double progress = (double) currDoc / (double) rowCount;
			exec.setProgress(progress, "Tagging document " + currDoc + " of "
					+ rowCount);
			currDoc++;

			// get current row
			DataRow row = it.next();
			// get document cell from row at index of document column
			DocumentValue docVal = (DocumentValue) row.getCell(m_docColIndex);
			
			// tag document and add tagged document to data table builder
			m_dtBuilder.addDocument(tagger.tag(docVal.getDocument()));
		}

		// close data table builder and create output data table
		return new BufferedDataTable[] { m_dtBuilder.getAndCloseDataTable() };
	}

    /**
     * {@inheritDoc}
     */
    @Override
    protected void reset() {
        // TODO: generated method stub
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void saveSettingsTo(final NodeSettingsWO settings) {
    	m_unmodifiableModel.saveSettingsTo(settings);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void loadValidatedSettingsFrom(final NodeSettingsRO settings)
            throws InvalidSettingsException {
    	m_unmodifiableModel.loadSettingsFrom(settings);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void validateSettings(final NodeSettingsRO settings)
            throws InvalidSettingsException {
    	m_unmodifiableModel.validateSettings(settings);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void loadInternals(final File internDir,
            final ExecutionMonitor exec) throws IOException,
            CanceledExecutionException {
        // Nothing to do ...
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void saveInternals(final File internDir,
            final ExecutionMonitor exec) throws IOException,
            CanceledExecutionException {
        // Nothing to do ...
    }
}

